@@ -1,5 +1,5 @@
 /* vi:set expandtab sw=2 sts=2: */
-/*-
+/*
  * Copyright (c) 2008 Jannis Pohlmann <jannis@xfce.org>
  *
  * This library is free software; you can redistribute it and/or
@@ -43,9 +43,6 @@ static gboolean xfce_shortcut_dialog_key_pressed      (XfceShortcutDialog      *
                                                        GdkEventKey             *event);
 static gboolean xfce_shortcut_dialog_key_released     (XfceShortcutDialog      *dialog,
                                                        GdkEventKey             *event);
-static gchar   *xfce_shortcut_dialog_shortcut_name    (XfceShortcutDialog      *dialog,
-                                                       guint                    keyval,
-                                                       guint                    modifiers);
 
 
 
@@ -196,35 +193,43 @@ xfce_shortcut_dialog_create_contents (XfceShortcutDialog *dialog,
                                       const gchar        *action_name,
                                       const gchar        *action)
 {
+  GtkWidget   *content_box;
+  GtkWidget   *alignment;
+  GtkWidget   *box;
   GtkWidget   *button;
-  GtkWidget   *table;
   GtkWidget   *label;
+  const gchar *action_type;
   const gchar *title;
-  const gchar *action_label;
+  const gchar *explanation_label;
+  gchar       *explanation_label_escaped;
+  gchar       *explanation_label_markup;
 
   if (g_utf8_collate (provider, "xfwm4") == 0)
     {
       title = _("Window Manager Action Shortcut");
-      action_label = _("Action:");
+      /* TRANSLATORS: this string will be used to create an explanation for
+       * the user in a following string */
+      action_type = _("action");
     }
   else if (g_utf8_collate (provider, "commands") == 0)
     {
       title = _("Command Shortcut");
-      action_label = _("Command:");
+      /* TRANSLATORS: this string will be used to create an explanation for
+       * the user in a following string */
+      action_type = _("command");
     }
   else
     {
       title = _("Shortcut");
-      action_label = _("Action:");
+      /* TRANSLATORS: this string will be used to create an explanation for
+       * the user in a following string */
+      action_type = _("action");
     }
 
   /* Set dialog title */
   gtk_window_set_title (GTK_WINDOW (dialog), title);
   gtk_window_set_icon_name (GTK_WINDOW (dialog), "input-keyboard");
 
-  /* Configure dialog */
-  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-
   /* Create clear button for xfwm4 */
   if (g_utf8_collate (provider, "xfwm4") == 0)
     {
@@ -238,31 +243,59 @@ xfce_shortcut_dialog_create_contents (XfceShortcutDialog *dialog,
   gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_CANCEL);
   gtk_widget_show (button);
 
-  table = gtk_table_new (2, 2, FALSE);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 12);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), table);
-  gtk_widget_show (table);
+  /* Main content container */
+  alignment = gtk_alignment_new (0, 0, 1, 1);
+  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 6, 12, 0);
+  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                     alignment);
+  gtk_widget_show (alignment);
 
-  label = gtk_label_new (action_label);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
-  gtk_widget_show (label);
+  #if GTK_CHECK_VERSION (3, 0, 0)
+  content_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  #else
+  content_box = gtk_vbox_new (FALSE, 6);
+  #endif
+  gtk_container_set_border_width (GTK_CONTAINER (content_box), 6);
+  gtk_container_add (GTK_CONTAINER (alignment), content_box);
+  gtk_widget_show (content_box);
 
-  label = gtk_label_new (action_name);
+  /* TRANSLATORS: this creates the explanation for the user. The first %s is replaced
+   * by the action type which you translated earlier, the second %s is replaced by the
+   * action name which comes from somewhere else.
+   * THE ORDER MUSTN'T BE REVERSED! */
+  explanation_label =
+    g_strdup_printf (_("Press now the keyboard keys you want to use to trigger the %s '%s'."),
+                     action_type, action_name);
+  explanation_label_escaped = g_markup_escape_text (explanation_label, -1);
+  explanation_label_markup = g_strdup_printf ("<i>%s</i>", explanation_label_escaped);
+
+  label = gtk_label_new (NULL);
+  gtk_label_set_markup (GTK_LABEL (label), explanation_label_markup);
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 0, 1);
+  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+  gtk_container_add (GTK_CONTAINER (content_box), label);
   gtk_widget_show (label);
+  g_free (explanation_label_escaped);
+  g_free (explanation_label_markup);
+
+  /* Box and labels to display the shortcut currently being grabbed.
+   * It will be updated to key-press events. */
+  #if GTK_CHECK_VERSION (3, 0, 0)
+  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+  #else
+  box = gtk_hbox_new (FALSE, 12);
+  #endif
+  gtk_container_add (GTK_CONTAINER (content_box), box);
+  gtk_widget_show (box);
 
   label = gtk_label_new (_("Shortcut:"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+  gtk_container_add (GTK_CONTAINER (box), label);
   gtk_widget_show (label);
 
-  dialog->shortcut_label = gtk_label_new (NULL);
+  dialog->shortcut_label = gtk_label_new (_("No keys pressed yet, proceed."));
   gtk_misc_set_alignment (GTK_MISC (dialog->shortcut_label), 0.0, 0.5);
-  gtk_table_attach_defaults (GTK_TABLE (table), dialog->shortcut_label, 1, 2, 1, 2);
+  gtk_container_add (GTK_CONTAINER (box), dialog->shortcut_label);
   gtk_widget_show (dialog->shortcut_label);
 
   /* Connect to key release signal for determining the new shortcut */
@@ -276,15 +309,46 @@ gint
 xfce_shortcut_dialog_run (XfceShortcutDialog *dialog,
                           GtkWidget          *parent)
 {
-  gint response = GTK_RESPONSE_CANCEL;
+#if GTK_CHECK_VERSION (3, 0, 0)
+  GdkDisplay       *display;
+  GdkDevice        *device;
+  GdkDeviceManager *device_manager;
+  GList            *devices, *li;
+  gboolean          succeed = FALSE;
+#endif
+  gint              response = GTK_RESPONSE_CANCEL;
 
   g_return_val_if_fail (XFCE_IS_SHORTCUT_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
   gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+  display = gtk_widget_get_display (GTK_WIDGET (dialog));
+  device_manager = gdk_display_get_device_manager (display);
+  devices = gdk_device_manager_list_devices (device_manager, GDK_DEVICE_TYPE_MASTER);
+
+  for (li = devices; li != NULL; li =li->next)
+    {
+      device = li->data;
+      if (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD)
+        continue;
+
+      if (gdk_device_grab (device, gtk_widget_get_root_window (GTK_WIDGET (dialog)),
+                           GDK_OWNERSHIP_WINDOW, TRUE,
+                           GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK,
+                           NULL, GDK_CURRENT_TIME) == GDK_GRAB_SUCCESS)
+        {
+          succeed = TRUE;
+        }
+    }
+
+  /* Take control on the keyboard */
+  if (succeed)
+#else
   /* Take control on the keyboard */
   if (G_LIKELY (gdk_keyboard_grab (gtk_widget_get_root_window (GTK_WIDGET (dialog)), TRUE, GDK_CURRENT_TIME) == GDK_GRAB_SUCCESS))
+#endif
     {
       /* Run the dialog and wait for the user to enter a valid shortcut */
       response = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -296,11 +360,27 @@ xfce_shortcut_dialog_run (XfceShortcutDialog *dialog,
           dialog->shortcut = g_strdup ("");
         }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+      /* Release keyboard */
+      for (li = devices; li != NULL; li =li->next)
+        {
+          device = li->data;
+          if (gdk_device_get_source (device) == GDK_SOURCE_KEYBOARD)
+            gdk_device_ungrab (device, GDK_CURRENT_TIME);
+        }
+#else
       /* Release keyboard */
       gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+#endif
     }
   else
-    g_warning ("%s", _("Could not grab the keyboard."));
+    {
+      g_warning (_("Could not grab the keyboard."));
+    }
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+  g_list_free (devices);
+#endif
 
   /* Return the response ID */
   return response;
@@ -312,21 +392,56 @@ static gboolean
 xfce_shortcut_dialog_key_pressed (XfceShortcutDialog *dialog,
                                   GdkEventKey        *event)
 {
-  gchar *text;
-  gchar *shortcut;
+  GdkKeymap       *keymap;
+  GdkModifierType  consumed, modifiers;
+  guint            keyval, mod_mask;
+  gchar           *text;
+  gchar           *escaped_label;
+  gchar           *label;
 
   g_free (dialog->shortcut);
 
-  /* Determine and remember the current shortcut */
-  dialog->shortcut = xfce_shortcut_dialog_shortcut_name (dialog, event->keyval, event->state);
+  /* Get the keyboard state */
+  mod_mask = gtk_accelerator_get_default_mod_mask ();
+  keymap = gdk_keymap_get_default ();
+  modifiers = event->state;
 
-  shortcut = g_markup_escape_text (dialog->shortcut, -1);
-  text = g_strdup_printf ("<span size='large'><b>%s</b></span>", shortcut);
+  gdk_keymap_translate_keyboard_state (keymap, event->hardware_keycode,
+                                       modifiers, event->group,
+                                       &keyval, NULL, NULL, &consumed);
+
+  /* We want Alt + Print to be Alt + Print not SysReq. See bug #7897 */
+  if (keyval == GDK_KEY_Sys_Req && (modifiers & GDK_MOD1_MASK) != 0)
+    {
+      consumed = 0;
+      keyval = GDK_KEY_Print;
+    }
+
+  /* Get the modifiers */
+
+  /* If Shift was used when translating the keyboard state, we remove it
+   * from the consumed bit because gtk_accelerator_{name,parse} fail to
+   * handle this correctly. This allows us to have shortcuts with Shift
+   * as a modifier key (see bug #8744). */
+  if ((modifiers & GDK_SHIFT_MASK) && (consumed & GDK_SHIFT_MASK))
+    consumed &= ~GDK_SHIFT_MASK;
+
+  modifiers &= ~consumed;
+  modifiers &= mod_mask;
+
+  /* Get and store the pressed shortcut */
+  dialog->shortcut = gtk_accelerator_name (keyval, modifiers);
+
+  label = gtk_accelerator_get_label (keyval, modifiers);
+  escaped_label = g_markup_escape_text (label, -1);
+  text = g_strdup_printf ("<span size='large'><b>%s</b></span>",
+                          escaped_label);
 
   gtk_label_set_markup (GTK_LABEL (dialog->shortcut_label), text);
 
+  g_free (label);
+  g_free (escaped_label);
   g_free (text);
-  g_free (shortcut);
 
   return FALSE;
 }
@@ -345,8 +460,10 @@ xfce_shortcut_dialog_key_released (XfceShortcutDialog *dialog,
   /* Check if the shortcut was accepted */
   if (G_LIKELY (shortcut_accepted))
     {
+#if !GTK_CHECK_VERSION (3, 0, 0)
       /* Release keyboard */
       gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+#endif
 
       /* Exit dialog with positive response */
       gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
@@ -354,7 +471,8 @@ xfce_shortcut_dialog_key_released (XfceShortcutDialog *dialog,
   else
     {
       /* Clear label */
-      gtk_label_set_markup (GTK_LABEL (dialog->shortcut_label), "");
+      gtk_label_set_markup (GTK_LABEL (dialog->shortcut_label),
+                            _("No keys pressed yet, proceed."));
     }
 
   return FALSE;
@@ -362,87 +480,6 @@ xfce_shortcut_dialog_key_released (XfceShortcutDialog *dialog,
 
 
 
-static gchar *
-xfce_shortcut_dialog_shortcut_name (XfceShortcutDialog *dialog,
-                                    guint               keyval,
-                                    guint               modifiers)
-{
-  XModifierKeymap *modmap;
-  Display         *display;
-  const KeySym    *keysyms;
-  KeyCode          keycode;
-  KeySym          *keymap;
-  gint             keysyms_per_keycode = 0;
-  gint             min_keycode = 0;
-  gint             max_keycode = 0;
-  gint             mask;
-  gint             i;
-  gint             j;
-
-  g_return_val_if_fail (XFCE_IS_SHORTCUT_DIALOG (dialog), NULL);
-
-  display = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
-
-  gdk_error_trap_push ();
-
-  XDisplayKeycodes (display, &min_keycode, &max_keycode);
-
-  keymap = XGetKeyboardMapping (display, min_keycode, max_keycode - min_keycode + 1, &keysyms_per_keycode);
-
-  if (G_LIKELY (keymap != NULL))
-    {
-      modmap = XGetModifierMapping (display);
-
-      if (G_LIKELY (modmap != NULL))
-        {
-          for (i = 0; i < 8 * modmap->max_keypermod; ++i)
-            {
-              keycode = modmap->modifiermap[i];
-
-              if (keycode == 0 || keycode < min_keycode || keycode > max_keycode)
-                continue;
-
-              keysyms = keymap + (keycode - min_keycode) * keysyms_per_keycode;
-              mask = 1 << (i / modmap->max_keypermod);
-
-              for (j = 0; j < keysyms_per_keycode; ++j)
-                {
-                  if (keysyms[j] == GDK_Super_L || keysyms[j] == GDK_Super_R)
-                    modifiers &= ~mask;
-
-#if 0
-                  if (keysyms[j] == GDK_Meta_L || keysyms[j] == GDK_Meta_R)
-                    modifiers &= ~mask;
-#endif
-
-                  if (keysyms[j] == GDK_Hyper_L || keysyms[j] == GDK_Hyper_R)
-                    modifiers &= ~mask;
-
-                  if (keysyms[j] == GDK_Scroll_Lock)
-                    modifiers &= ~mask;
-
-                  if (keysyms[j] == GDK_Num_Lock)
-                    modifiers &= ~mask;
-
-                  if (keysyms[j] == GDK_Caps_Lock)
-                    modifiers &= ~mask;
-                }
-            }
-
-          XFreeModifiermap (modmap);
-        }
-
-      XFree (keymap);
-    }
-
-  gdk_flush ();
-  gdk_error_trap_pop ();
-
-  return gtk_accelerator_name (keyval, modifiers);
-}
-
-
-
 const gchar*
 xfce_shortcut_dialog_get_shortcut (XfceShortcutDialog *dialog)
 {
